#!/usr/bin/env /python
"""
SinGen+Fit
Generate sine curve with noise, then fit using Levenberg-Marquardt 
method.
"""
import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize
from scipy.optimize import leastsq

# Generate the data
npts = 330
amp = 2.
per = 10.
T0  = 1.0
noise_amp = 0.5

t = np.linspace(0.,50.,npts)
y = amp*np.sin(2*np.pi/per*(t-T0))+noise_amp*np.random.randn(npts)

fitfunc = lambda p, t: p[1]*np.sin(2*np.pi/p[0]*(t-p[2]))
errfunc = lambda p, t, y: fitfunc(p,t) - y

p0 = [9.,1.,3.]  # Initial guesses for fit parameters
yguess = p0[1]*np.sin(2*np.pi/p0[0]*(t-p0[2]))
p1,cov,info,mesg, success = optimize.leastsq(errfunc, p0,args=(t,y),
                                             full_output=True)

chisq=sum(info["fvec"]*info["fvec"])
dof = len(t) - len(p1)
rmsred = np.sqrt(chisq/dof)
print "Converged with chi squared ",chisq
print "degrees of freedom, dof    ",dof
print "sqrt(chisq/dof)            ",rmsred

print "Best Fit, with 1-sigma errors:"
print "P  = %.3f +- %.3f" % (p1[0],np.sqrt(cov[0,0])*np.sqrt(chisq/dof))
print "A  = %.3f +- %.3f" % (p1[1],np.sqrt(cov[1,1])*np.sqrt(chisq/dof))
print "T0 = %.3f +- %.3f" % (p1[2],np.sqrt(cov[2,2])*np.sqrt(chisq/dof))
    
plt.plot(t,y,"ro",t,fitfunc(p1,t),"r-")
plt.plot(t,yguess,"g-")
plt.ylim(-4,5.5)
plt.xlim(-1,51)

plt.xlabel("Time (s)")
plt.ylabel("Amplitude")
plt.legend(('Data','Fit','Init Params'))

ax = plt.axes()
s = 'Best Fit:\n P  = %.3f\n A  = %.3f\n T0 = %.3f'%(p1[1],p1[0],p1[2])
plt.text(2.0, 3.5,s, family='monospace',fontsize=12)

plt.show()
